// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Drawing;
using System.Numerics;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // このサンプルは、GcGraphics.PushTransparencyLayer()／PopTransparencyLayer() メソッドを使用して、
    // 複数の描画操作に対して共通の不透明度を指定する方法を示しています。
    // コードは AddWatermark のサンプルと似ていますが、半透明の文字色を使用する代わりに、
    // 透明レイヤーに関連するすべての描画を行っています。
    // これにより、より柔軟で便利な透過性の制御を実現しています。
    public class TransparencyLayer
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            using var fs = File.OpenRead(Path.Combine("Resources", "PDFs", "SlidePages.pdf"));
                doc.Load(fs);
                foreach (var page in doc.Pages)
                {
                    var g = page.Graphics;

                    // 透かしの描画に使用されるテキストレイアウト。
                    var tl = g.CreateTextLayout();
                    tl.Append("DioDocs Demo");
                    tl.DefaultFormat.Font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "calibrib.ttf"));
                    tl.DefaultFormat.FontSize = g.Resolution;
                    tl.DefaultFormat.ForeColor = Color.Blue;
                    tl.DefaultFormat.GlyphAdvanceFactor = 1.5f;
                    tl.PerformLayout();

                    // 回転角度（ラジアン） - ページの左下から右上の角まで。
                    var angle = -Math.Asin(g.CanvasSize.Width / g.CanvasSize.Height);
                    // ページの中央。
                    var center = new PointF(g.CanvasSize.Width / 2, g.CanvasSize.Height / 2);
                    // テキストサイズを基にオフセットを追加。
                    var delta = new PointF(
                        (float)((tl.ContentWidth * Math.Cos(angle) - tl.ContentHeight * Math.Sin(angle)) / 2),
                        (float)((tl.ContentWidth * Math.Sin(angle) + tl.ContentHeight * Math.Cos(angle)) / 2));

                    // 半透明の透明レイヤーを作成します。
                    // 以降のすべての描画操作は、PopTransparencyLayer() メソッドが呼び出されるまで、
                    // 指定された不透明度を使用して描画されます。
                    g.PushTransparencyLayer(null, 0.5f);

                    // 透かしテキストをページの中央に斜めに描画します。
                    // （Matrixの変換は最後から最初へ適用されます）
                    g.Transform =
                        Matrix3x2.CreateRotation((float)angle) *
                        Matrix3x2.CreateTranslation(center.X - delta.X, center.Y - delta.Y);
                    g.DrawTextLayout(tl, PointF.Empty);
                    // テキストの周りに枠線を追加します。これも半透明で描画されます。
                    var rc = new RectangleF(-30, -20, tl.ContentWidth + 60, tl.ContentHeight + 40);
                    g.DrawRoundRect(rc, 30, new GCDRAW.Pen(Color.DarkOrange, 8));
                    g.Transform = Matrix3x2.Identity;

                    // PopTransparencyLayer() メソッドを呼び出し。
                    g.PopTransparencyLayer();
                }
                doc.Save(stream);
            return doc.Pages.Count;
        }
    }
}
